package Yarssr::GUI;

use Gtk2;
use Gtk2::GladeXML;
use Gtk2::SimpleList;
use Gtk2::TrayIcon;
use Gnome2;
use Yarssr::Config;

use constant TRUE=>1,FALSE=>0;

my $prefs_window;
my $add_dialog;
my $prop_dialog;
my $gld;
my $menu;
my $pref_menu;
my $menu_x;
my $menu_y;
my $icon;
my $eventbox;
my $tooltips;

my $paper_grey_pixbuf;
my $paper_red_pixbuf;
my $paper_green_pixbuf;
my $dot_full_red_pixbuf;
my $dot_hollow_red_pixbuf;
my $blank_pixbuf;
my $tray_image;

sub init {
	my $class = shift;

	Gtk2->init;

	my $imagedir = $Yarssr::PREFIX."/share/yarssr/pixmaps/";
	
	$paper_grey_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
	    $imagedir."paper_grey.xpm");
	$paper_red_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
	    $imagedir."paper_red.xpm");
	$paper_green_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
	    $imagedir."paper_green.xpm");
	$dot_full_red_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
	    $imagedir."full_red.xpm");
	$dot_hollow_red_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
	    $imagedir."hollow_red.xpm");
	$blank_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
	    $imagedir."blank.xpm");
	$tray_image = Gtk2::Image->new_from_pixbuf(
	    $paper_grey_pixbuf);

	$tooltips = Gtk2::Tooltips->new;

	# Create tray inon
	my $tray = Gtk2::TrayIcon->new("rss");
	$eventbox = Gtk2::EventBox->new;
	
	$eventbox->add($tray_image);
	set_icon_active();	

	$tray->add($eventbox);
	$tray->show_all;
	Gtk2->main_iteration while Gtk2->events_pending;	
	
	$gld = Gtk2::GladeXML->new($Yarssr::PREFIX.'/share/yarssr/yarssr.glade');
	$gld->signal_autoconnect_from_package('main');
	
	$prefs_window = $gld->get_widget('window_prefs');
	$prefs_window->signal_connect('delete-event'=>\&delete_event);

	$feedinfo_dialog = $gld->get_widget('feed_dialog');
	$feedinfo_dialog->set_transient_for($prefs_window);
	$feedinfo_dialog->signal_connect('delete-event'=>\&delete_event);

	$feedlist = Gtk2::SimpleList->new (
		''			=> 'pixbuf',
		'Name'		=> 'text',
		'Enabled'	=> 'bool',
		'Address'	=> 'text',
	);

	$feedlist->set_rules_hint(1);
	$feedlist->get_selection->set_mode ('multiple');

	my @columns = $feedlist->get_columns;
	$columns[2]->set_clickable(1);
	$columns[2]->signal_connect('clicked',\&enabled_header_clicked);

	# Add simplelist to scrolled window in prefs window
	my $scrolledwindow = $gld->get_widget('scrolledwindow_feeds');
	$scrolledwindow->add($feedlist);

	create_prefs_menu();

	Gtk2->main;
};

sub delete_event {
    $_[0]->hide;
    feedinfo_dialog_clear();
    return 1;
}

sub set_icon_active {
    	$eventbox->signal_handlers_disconnect_by_func(\&handle_button_press);
	$eventbox->signal_connect("button-press-event", \&ignore_button_press);
	$tray_image->set_from_pixbuf($paper_red_pixbuf);
	set_tooltip("updating...");
	gui_update();
}

sub set_icon_inactive {
    $eventbox->signal_handlers_disconnect_by_func(\&ignore_button_press);
    $eventbox->signal_connect("button-press-event", \&handle_button_press);
    
    my $newitems = Yarssr->get_total_newitems;

    if ($newitems) {
		$tray_image->set_from_pixbuf($paper_green_pixbuf);
    } else {
		$tray_image->set_from_pixbuf($paper_grey_pixbuf);
    }
    
    set_tooltip($newitems." new links since last update");
    gui_update();
}

sub launch_url {
	my $url = shift;

	if (Yarssr::Config->get_usegnome) {
		Gnome2::URL->show($url);
	}
	else {
		if (fork == 0)
		{
			my $b = Yarssr::Config->get_browser;
			$b .= " \"$url\"" unless $b =~ s/\%s/"$url"/;
			exec($b) or warn "unable to launch browser\n";
			
			#$b .= ' %s' unless $b =~ /\%s/;
			#exec(sprintf($b,$url)) or
			#	warn "Unable to launch browser!\n";
			exit;
		}
	}
}

sub prefs_show
{
    @{$feedlist->{data}} = ();

    for (Yarssr->get_feeds_array) {
		my $pixbuf = undef;
		if ($_->get_status) {
			my $invis = Gtk2::Invisible->new;
			$pixbuf = $invis->render_icon('gtk-dialog-warning','menu')
		}
		push @{$feedlist->{data}}, [$pixbuf,$_->get_title,$_->get_enabled,$_->get_url];
    }
    
    $gld->get_widget('interval_entry')->set_text(Yarssr::Config->get_interval);
    $gld->get_widget('headings_entry')->set_text(Yarssr::Config->get_maxfeeds);
    $gld->get_widget('browser_entry')->set_text(Yarssr::Config->get_browser);
    $gld->get_widget('browser_entry')->set_sensitive(
		!Yarssr::Config->get_usegnome);
    $gld->get_widget('use_default_browser_checkbox')->set_active(
		Yarssr::Config->get_usegnome);
	$gld->get_widget('start_online_checkbutton')->set_active(
		Yarssr::Config->get_startonline);

    $gld->get_widget('pref_ok_button')->grab_focus;
    $prefs_window->show_all;
}

sub on_pref_cancel_button_clicked
{
	$prefs_window->hide;
	$feedinfo_dialog->hide;
}

sub on_pref_ok_button_clicked
{
	$prefs_window->hide;
	Gtk2->main_iteration while Gtk2->events_pending;	

	my $interval = $gld->get_widget('interval_entry')->get_text;
	my $maxfeeds = $gld->get_widget('headings_entry')->get_text;
	my $browser = $gld->get_widget('browser_entry')->get_text;
	my $online = $gld->get_widget('start_online_checkbutton')->get_active;
	my $usegnome;
	
	if ($gld->get_widget('use_default_browser_checkbox')->get_active) {
	    $usegnome = 1;
	}
	else {
	    $usegnome = 0;
	}

	my $newfeedlist;
	for (@{$feedlist->{data}}) {
	    $newfeedlist->{$_->[3]} = [$_->[1],$_->[2]];
	}
	set_icon_active();
	redraw_menu() if Yarssr::Config->process(
		$interval,$maxfeeds,$browser,$usegnome,$newfeedlist,$online);
    Yarssr::Config->write_config;
    set_icon_inactive();
}


sub on_use_default_browser_checkbox_toggled
{
	my ($widget, $window) = @_;

	my $browser_entry = $gld->get_widget('browser_entry');

	if ($widget->get_active)
	{
		$browser_entry->set_sensitive(0);
	}
	else
	{
		$browser_entry->set_sensitive(1);
	}
}

sub on_add_button_clicked
{
    my $ok_button = $gld->get_widget('feedinfo_ok_button');
	$ok_button->signal_handlers_disconnect_by_func(\&properties_change);
	
	$ok_button->signal_connect('clicked',\&feedinfo_add);
	
    $feedinfo_dialog->show_all;
}

sub on_remove_button_clicked {
    my @selected = $feedlist->get_selected_indices;
    for my $row (reverse @selected) {
		splice @{$feedlist->{data}},$row,1;
    }
}

sub feedinfo_add {
    my $title = $gld->get_widget('feedinfo_name');
    my $url = $gld->get_widget('feedinfo_address');
	my $ok_button = $gld->get_widget('feedinfo_ok_button');

	$ok_button->signal_handlers_disconnect_by_func(\&feedinfo_add);
    
    unless (Yarssr->get_feed_by_url($url->get_text) or 
 	   Yarssr->get_feed_by_title($title->get_text)) {
    	push @{$feedlist->{data}},[undef,$title->get_text,1,$url->get_text];
    }

    feedinfo_dialog_clear();
    $feedinfo_dialog->hide;
}

sub feedinfo_dialog_clear {
    $gld->get_widget('feedinfo_name')->set_text('');
    $gld->get_widget('feedinfo_address')->set_text('');
}


sub on_feedinfo_cancel_button_clicked {
    feedinfo_dialog_clear();
    $feedinfo_dialog->hide;
}

sub redraw_menu {
	my $class = shift;
	Yarssr->log_debug("Rebuilding menu");

	for my $feed (Yarssr->get_feeds_array) {
		Gtk2->main_iteration while Gtk2->events_pending;	
		create_feed_menu($feed) if $feed->get_enabled;
	}
	create_root_menu();
	set_icon_inactive();
}

sub create_root_menu {
	$menu = Gtk2::Menu->new;
	my $refresh = Gtk2::ImageMenuItem->new_from_stock('gtk-refresh');
	my $clear_new = Gtk2::ImageMenuItem->new("Unmark new");
	$clear_new->set_image(Gtk2::Image->new_from_stock('gtk-clear','menu'));
	$refresh->signal_connect('activate',sub {
		$menu->popdown;
		Gtk2->main_iteration while Gtk2->events_pending;	
		Yarssr->download_all;
	});
	$clear_new->signal_connect('activate',sub {
		Yarssr->clear_newitems;
		redraw_menu();
	});
	$menu->append($refresh);
	$menu->append($clear_new);
	$menu->append(Gtk2::SeparatorMenuItem->new);
	
	for my $feed (Yarssr->get_feeds_array) {
		if ($feed->get_enabled) {
		   	my $title = $feed->get_title;
			my $menuitem = Gtk2::ImageMenuItem->new($title);
			$menuitem->child->set_markup("<b>$title</b>") if $feed->get_newitems;

			if ($feed->get_status == 0 and $feed->get_icon) {
				my $image = Gtk2::Image->new_from_pixbuf($feed->get_icon);
				$menuitem->set_image($image);
		    }
			elsif ($feed->get_status) {
				my $image = Gtk2::Image->new_from_stock('gtk-dialog-warning','menu');
				$menuitem->set_image($image);
			}

			$menuitem->set_submenu($feed->get_menu);
			$menu->append($menuitem);
		}
	}
	$menu->show_all;
	return $menu;
}

sub create_prefs_menu {
    $pref_menu = Gtk2::Menu->new;
    my $quit = Gtk2::ImageMenuItem->new_from_stock('gtk-quit');
    my $prefs = Gtk2::ImageMenuItem->new_from_stock('gtk-preferences');
    my $about = Gtk2::ImageMenuItem->new("About");

	my $online;
	my $imageroot = $Yarssr::PREFIX."/share/yarssr/pixmaps/";

	if (Yarssr::Config->get_online) {
		$online = Gtk2::ImageMenuItem->new("Go Offline");
		my $image = Gtk2::Image->new_from_file($imageroot."disconnect.png");
		$online->set_image($image);
		$online->signal_connect('activate',
			sub { Yarssr::Config->set_online(0); create_prefs_menu(); });
		
	}
	else {
		$online = Gtk2::ImageMenuItem->new("Go Online");
		my $image = Gtk2::Image->new_from_file($imageroot."connect.png");
		$online->set_image($image);
		$online->signal_connect('activate',
			sub { Yarssr::Config->set_online(1); create_prefs_menu(); });
	}
	
    $about->set_image(Gtk2::Image->new_from_stock('gnome-stock-about','menu'));
    $about->signal_connect('activate',\&on_about_button_clicked);
    $prefs->signal_connect('activate',\&prefs_show);
    $quit->signal_connect('activate',sub { 
	    Yarssr::Config->write_config;
	    Yarssr::Config->write_states;
	    exit;});
    $pref_menu->append($online);
    $pref_menu->append($prefs);
    $pref_menu->append($about);
    $pref_menu->append($quit);
    $pref_menu->show_all;
    return $prefs_menu;
}

sub create_feed_menu {
	my $feed = shift;
	$feed->new_menu;
	my $feedcounter = 0;
	my $image;
	$feed->reset_newitems;
	foreach my $item ($feed->get_items_array) {
		Gtk2->main_iteration while Gtk2->events_pending;
		last if $feedcounter == Yarssr::Config->get_maxfeeds;
		$feedcounter++;

		# Clean up the title for the menu
		my $title;
		$title = $item->get_title or $title = '-no title-';
		$title =~ s/\n//g;
		$title =~ s/(?:\s+|\t+)/ /g;
		my @title = split /(.{50}.+?)\s/,$title;
		shift @title if $title[0] eq '';
		$title = shift @title;
		foreach (@title) {
			next if $_ eq '';
			$title .= "\n";
			$title .= $_;
		}
		
		my $menuitem = Gtk2::ImageMenuItem->new($title);
		my $status = $item->get_status;
		
		if ($status == 4 or $status == 3) {
			$image = Gtk2::Image->new_from_pixbuf(
			    $dot_full_red_pixbuf);
			$menuitem->set_image($image);
			$item->set_status(3);
			$feed->add_newitem;
		}
		elsif ($status == 2) {
		    $image = Gtk2::Image->new_from_pixbuf(
			$dot_hollow_red_pixbuf);
		    $menuitem->set_image($image);
		}
		else {
			$image = Gtk2::Image->new_from_pixbuf(
			    $blank_pixbuf);
			$item->set_status(1);
			$menuitem->set_image($image);
		}
		$menuitem->signal_connect('activate',sub {
			menuitem_clicked($menuitem,$item);
		    });
		$feed->get_menu->append($menuitem);
	}
	$feed->get_menu->append(Gtk2::SeparatorMenuItem->new);
	my $update = Gtk2::ImageMenuItem->new("Update this feed");
	$update->set_image(Gtk2::Image->new_from_stock('gtk-refresh','menu'));
	$update->signal_connect('activate',sub{ 
			set_icon_active();
			Yarssr->download_feed($feed);
			redraw_menu();
			set_icon_inactive();
		});
	$feed->get_menu->append($update);
	$feed->get_menu->show_all;
}	

sub menuitem_clicked {
	my $menuitem = shift;
	my $item = shift;
	
	launch_url($item->get_url);

	my $feed = $item->get_parent;
	my $status = $item->get_status;

	unless ($status == 1) {
		Glib::Idle->add(sub {
				Yarssr::Config->write_state($feed);
			});
	}
	
	my $newitems;
	
	if ($status > 2) {
	    $newitesms = $feed->subtract_newitem();
	}

	$item->set_status(1);
	redraw_menu() unless $newitems;
	
	set_tooltip(Yarssr->get_total_newitems." new links since last update");
	
	my $image = Gtk2::Image->new_from_pixbuf($blank_pixbuf);
	$menuitem->set_image($image);
}

sub get_menu {
	return $menu->{menu};
}

sub ignore_button_press {
	Yarssr->log_debug("Menu is disabled while updating");
    return 1;
}

sub handle_button_press {
    my $widget = shift;
    my $event = shift;
    
    $menu_x = $event->x_root - $event->x;
    $menu_y = $event->y_root - $event->y;
    
    if ($event->button == 1) {
	$menu->popup(undef,undef,\&position_menu,0,$event->button,$event->time)
    }
    else {
	$pref_menu->popup(undef,undef,\&position_menu,0,$event->button,$event->time);
    }
}

sub position_menu {

    # Shamlessly stolen from Muine :-)
    
    my $x = $menu_x;
    my $y = $menu_y;

    my $monitor = $menu->get_screen->get_monitor_at_point($x,$y);
    my $rect = $menu->get_screen->get_monitor_geometry($monitor);

    my $space_above = $y - $rect->y;
    my $space_below = $rect->y + $rect->height - $y;

    my $requisition = $menu->size_request();

    if ($requisition->height <= $space_above ||
	$requisition->height <= $space_below) {
	
	if ($requisition->height <= $space_below) {
	    $y = $y + $eventbox->allocation->height; 
	}
	
	else {
	    $y = $y - $requisition->height;
	}
	
    }
    
    elsif ($requisition->height > $space_below and
	$requisition->height > $space_above) {
	
	if ($space_below >= $space_above) {
	    $y = $rect->y + $rect->height - $requisition->height;
	}

	else {
	    $y = $rect->y;
	}
    }
    
    else {
	$y = $rect->y;
    }
    return ($x,$y,1);
}

sub gui_update {
    Gtk2->main_iteration while Gtk2->events_pending;	
}

sub on_about_button_clicked {
    my $logo = $paper_grey_pixbuf->scale_simple(64,64,'tiles');
    my $author = "$Yarssr::AUTHOR\n\n Patches from:\n";
    $author .= "\t$_\n" for @Yarssr::COAUTHORS;
	$author .= "\n$_" for @Yarssr::TESTERS;
    my $about = Gnome2::About->new(
		$Yarssr::NAME,$Yarssr::VERSION,$Yarssr::LICENSE,
		$Yarssr::URL,$author,undef,undef,$logo);
    $about->show;
}

sub on_properties_button_clicked {
    my @selected = $feedlist->get_selected_indices;
    return unless @selected;
    my $row = $feedlist->{'data'}[$selected[0]];
    
    $gld->get_widget('feedinfo_name')->set_text($row->[1]);
    $gld->get_widget('feedinfo_address')->set_text($row->[3]);

	my $ok_button = $gld->get_widget('feedinfo_ok_button');

    $ok_button->signal_handlers_disconnect_by_func(\&feedinfo_add);

	$ok_button->signal_connect('clicked', \&properties_change,$row);

    $feedinfo_dialog->show_all;
}

sub properties_change {
    my (undef,$row) = @_;
	my $feed = Yarssr->get_feed_by_url($row->[2]);
	my $ok_button = $gld->get_widget('feedinfo_ok_button');
	
	$ok_button->signal_handlers_disconnect_by_func(\&properties_change);
	$feedinfo_dialog->hide;
    my $new_title = $gld->get_widget('feedinfo_name')->get_text;
    my $new_url = $gld->get_widget('feedinfo_address')->get_text;
    
    $row->[1] = $new_title;
    $row->[3] = $new_url;

    # Need to figure out some way to apply this only when
    # the user clicks Ok.
	if ($feed) {
	    $feed->set_title($new_title);
		$feed->set_url($new_url);
		$feed->enable_and_flag if $feed->get_enabled;
	}

	feedinfo_dialog_clear();
}

sub enabled_header_clicked {
    my $total = @{$feedlist->{data}};
    my $enabled = 0;
    for (@{$feedlist->{'data'}}) {
		$enabled += $_->[2];
    }
    
    my $bool = 1;
    $bool = 0 if $total == $enabled;

    for (@{$feedlist->{'data'}}) {
		$_->[2] = $bool;
    }
}

sub set_tooltip {
    my $text = shift;
    $tooltips->set_tip($eventbox,$text);
}

sub quit {
	Gtk2->main_quit;
}

1;
